//
//  SBSDKUIBarcodesBatchScannerConfiguration.swift
//  barcode_scanner
//
//  Created by Yevgeniy Knizhnik on 20.01.21.
//

import Foundation
import ScanbotBarcodeScannerSDK

extension SBSDKUIBarcodesBatchScannerConfiguration {
    convenience init(from dictionary: [String: Any]) {
        let uiConfig = SBSDKUIBarcodesBatchScannerUIConfiguration(from: dictionary)
        let textConfig = SBSDKUIBarcodesBatchScannerTextConfiguration(from: dictionary)
        let behaviourConfig = SBSDKUIBarcodesBatchScannerBehaviorConfiguration(from: dictionary)
        let cameraConfig = SBSDKUIBarcodesBatchScannerConfiguration.fixCameraConfigurationDeviceIssue(SBSDKUICameraConfiguration(from: dictionary))
        let selectionOverlayConfig = SBSDKUIBarcodeSelectionOverlayConfiguration(from: dictionary)
        
        self.init(uiConfiguration: uiConfig,
                  textConfiguration: textConfig,
                  behaviorConfiguration: behaviourConfig,
                  cameraConfiguration: cameraConfig,
                  selectionOverlayConfiguration: selectionOverlayConfig)
    }
    
    // -----------------------------------------------------------------------
    //  TODO: Remove after updating with iPhone 15 and 15 Pro fix.
    // -----------------------------------------------------------------------
    internal static func fixCameraConfigurationDeviceIssue(
        _ cameraConfiguration: SBSDKUICameraConfiguration
    ) -> SBSDKUICameraConfiguration {
        if let camera = cameraConfiguration.camera, camera.devicePosition == .back {
            
            var device: SBSDKCameraDevice?
            let deviceName = self.deviceName
            
            if deviceName == "iPhone16,1" || deviceName == "iPhone16,2" {
                device = SBSDKCameraDevice.availableDevices(for: .ultraWide, andPosition: .back).first
            }
            
            if let ultraWideDevice = device {
                cameraConfiguration.camera = ultraWideDevice
            } else if let defaultDevice = SBSDKCameraDevice.defaultBackFacingCamera() {
                cameraConfiguration.camera = defaultDevice
            }
        }
        return cameraConfiguration
    }
    
    // -----------------------------------------------------------------------
    //  TODO: Remove after updating with iPhone 15 and 15 Pro fix.
    // -----------------------------------------------------------------------
    internal static let deviceName: String = {
        var systemInfo = utsname()
        uname(&systemInfo)
        let modelCode = withUnsafePointer(to: &systemInfo.machine) {
            $0.withMemoryRebound(to: CChar.self, capacity: 1) {
                ptr in String.init(validatingUTF8: ptr)
            }
        }
        return modelCode!
    }()
}
